==> 0.0.2.tar.gz: lib/services/cube.dart <==
import 'package:bizdoc/models/models.dart';
import 'package:bizdoc/models/profile.dart';
import 'package:bizdoc/services/session.dart';

import 'base.dart';
import 'datasource.dart';

const complexAxisDivider = '__';

class Cube extends Service {
  Future<List<CubeSeries>> series(String cube, List<String> xAxis,
      {List<String>? series,
      List<String>? indices,
      Map<String, dynamic>? filters}) async {
    final response = await dio.put('/cube/$cube', data: {
      'series': series,
      'xAxis': xAxis,
      'filters': filters,
      'indices': indices
    });
    final results =
        List<CubeSeries>.from(response.data.map((e) => CubeSeries.fromJson(e)));
    if (results.isEmpty) return results;
    final base = results.elementAt(0);
    final columns = xAxis.toSet();
    if (series != null) columns.addAll(series);
    final configuration =
        session.profile.cubes.firstWhere((element) => element.name == cube);
    final data = await _prepare(cube, columns);

    String translate(String key, List<String> axes) {
      final keys = key.split(complexAxisDivider), labels = [];
      for (var i = 0; i < keys.length; i++) {
        final val = data[axes[i]]![keys[i]];
        labels.add(val ?? keys[i]);
      }
      return labels.join('-');
    }

    var i = 0;
    for (var item in results) {
      for (var point in item.points) {
        point.xLabel = translate(point.x, xAxis);
        if (i > 0 && !base.points.any((element) => element.x == point.x)) {
          base.points.add(Point(point.x, 0));
        }
      }
      if (item.name != null) {
        item.title = translate(item.name!, series!);
      } else {
        final index = configuration.indices
            .firstWhere((element) => element.name == item.index);
        item.title = index.title;
      }
      i++;
    }
    return results;
  }

  ///
  Future<Map<String, Map<String, dynamic>>> _prepare(
      String cube, Iterable<String> columns) async {
    final Map<String, Map<String, dynamic>> map = {};
    final ccube =
        session.profile.cubes.firstWhere((element) => element.name == cube);
    final axes = ccube.axes.where((element) => columns.contains(element.name));
    await Future.wait(axes.map((axis) {
      final name = axis.name;
      return dataSource.getMap(axis.dataType).then((datasource) {
        return map[name] = datasource;
      });
    }));
    return map;
  }

  ///
  Future<List<Map>> table(String cube, List<String> columns,
      {List<String>? indices, Map<String, dynamic>? filters}) async {
    final response = await dio.put('/cube/table/$cube', data: {
      'columns': columns,
      'indices': indices,
      'filters': filters,
    });
    final data = response.data as List<Map>;

    final datasources = await _prepare(cube, columns);

    for (var item in data) {
      for (var axis in columns) {
        item[axis + 'Label'] = datasources[axis]![item[axis]] ?? item[axis];
      }
    }

    return data;
  }

  Future<Map<String, Map<String, dynamic>>> constraints(String cube,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/constraints/$cube', queryParameters: axes);
    return response.data;
  }

  Future<List<Map<String, dynamic>>> explore(String cube,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/explore/$cube', queryParameters: axes);
    return response.data;
  }

  Future<ExploreType> resolve(String cube, {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/exploretype/$cube', queryParameters: axes);
    return ExploreType.fromJson(response.data);
  }

  Future<List<List<String>>> combinations(String cube,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/combinations/$cube', queryParameters: axes);
    return response.data;
  }

  Iterable<CurrencyInfo>? _currencies;
  Future<Iterable<CurrencyInfo>> currencies() async {
    if (_currencies != null) return _currencies!;
    final response = await dio.get('/cube/currencies');
    return _currencies =
        (response.data as List).map((e) => CurrencyInfo.fromJson(e));
  }

  // final Map<String, double> _exchangeRate = {};
  Future<double> exchangeRate(String code, {DateTime? date}) async {
    final response = await dio
        .get('/cube/exchangeRate/$code', queryParameters: {'date': date});
    return response.data ?? 1;
  }

  Future<Iterable<PatternInfo>> patterns(String cube) async {
    final response = await dio.get('/cube/patterns/$cube');
    return (response.data as List).map((e) => PatternInfo.fromJson(e));
  }

  Future<Map<String, dynamic>> segmentValues(String cube, String axis) async {
    final response = await dio.get('/cube/segmentvalues/$cube/$axis');
    return response.data;
  }
}

var cube = Cube();

==> 0.0.3.tar.gz: lib/services/cube.dart <==
import '../models/cube.dart';
import '../models/profile.dart';

import 'base.dart';
import 'session.dart';
import 'datasource.dart';

const complexAxisDivider = '__';

class Cube extends Service {
  Future<List<CubeSeries>> series(String cube, List<String> xAxis,
      {List<String>? series,
      List<String>? indices,
      Map<String, dynamic>? filters,
      bool? groupIndices,
      bool? inverseIndices}) async {
    final response = await dio.put('/cube/$cube', data: {
      'series': series,
      'xAxis': xAxis,
      'filters': filters,
      'indices': indices,
      'groupIndices': groupIndices,
      'inverseIndices': inverseIndices
    });
    final results =
        List<CubeSeries>.from(response.data.map((e) => CubeSeries.fromJson(e)));
    if (results.isEmpty) return results;
    final base = results.elementAt(0);
    final columns = xAxis.toSet();
    if (series != null) columns.addAll(series);
    final configuration =
        session.profile.cubes.firstWhere((element) => element.name == cube);
    final data = await _prepare(cube, columns);

    String translate(String key, List<String> axes) {
      final keys = key.split(complexAxisDivider), labels = [];
      for (var i = 0; i < keys.length; i++) {
        final val = data[axes[i]]![keys[i]];
        labels.add(val ?? keys[i]);
      }
      return labels.join('-');
    }

    var i = 0;
    for (var item in results) {
      for (var point in item.points) {
        point.xLabel = translate(point.x, xAxis);
        if (i > 0 && !base.points.any((element) => element.x == point.x)) {
          base.points.add(Point(point.x, 0));
        }
      }
      if (item.name != null) {
        item.title = translate(item.name!, series!);
      } else {
        final index = configuration.indices
            .firstWhere((element) => element.name == item.index);
        item.title = index.title;
      }
      i++;
    }
    return results;
  }

  ///
  Future<Map<String, Map<String, dynamic>>> _prepare(
      String cube, Iterable<String> columns) async {
    final Map<String, Map<String, dynamic>> map = {};
    final ccube =
        session.profile.cubes.firstWhere((element) => element.name == cube);
    final axes = ccube.axes.where((element) => columns.contains(element.name));
    await Future.wait(axes.map((axis) {
      final name = axis.name;
      return dataSource.map(axis.dataType).then((datasource) {
        return map[name] = datasource;
      });
    }));
    return map;
  }

  ///
  Future<List<Map>> table(String cube, List<String> columns,
      {List<String>? indices, Map<String, dynamic>? filters}) async {
    final response = await dio.put('/cube/table/$cube', data: {
      'columns': columns,
      'indices': indices,
      'filters': filters,
    });
    final data = response.data as List<Map>;

    final datasources = await _prepare(cube, columns);

    for (var item in data) {
      for (var axis in columns) {
        item[axis + 'Label'] = datasources[axis]![item[axis]] ?? item[axis];
      }
    }

    return data;
  }

  Future<Map<String, Map<String, dynamic>>> constraints(String cube,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/constraints/$cube', queryParameters: axes);
    return response.data;
  }

  Future<List<Map<String, dynamic>>> explore(String cube,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/explore/$cube', queryParameters: axes);
    return response.data;
  }

  Future<List<Map<String, dynamic>>> exploreIndex(String cube, String index,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/exploreIndex/$cube/$index', queryParameters: axes);
    return response.data;
  }

  Future<ExploreType> resolve(String cube, {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/exploretype/$cube', queryParameters: axes);
    return ExploreType.fromJson(response.data);
  }

  Future<List<List<String>>> combinations(String cube,
      {Map<String, dynamic>? axes}) async {
    final response =
        await dio.get('/cube/combinations/$cube', queryParameters: axes);
    return response.data;
  }

  Iterable<CurrencyInfo>? _currencies;
  Future<Iterable<CurrencyInfo>> currencies() async {
    if (_currencies != null) return _currencies!;
    final response = await dio.get('/cube/currencies');
    return _currencies =
        (response.data as List).map((e) => CurrencyInfo.fromJson(e));
  }

  // final Map<String, double> _exchangeRate = {};
  Future<double> exchangeRate(String code, {DateTime? date}) async {
    final response = await dio
        .get('/cube/exchangeRate/$code', queryParameters: {'date': date});
    return response.data ?? 1;
  }

  Future<Iterable<PatternInfo>> patterns(String cube) async {
    final response = await dio.get('/cube/patterns/$cube');
    return (response.data as List).map((e) => PatternInfo.fromJson(e));
  }

  Future<Map<String, dynamic>> segmentValues(String cube, String axis) async {
    final response = await dio.get('/cube/segmentValues/$cube/$axis');
    return response.data;
  }
}

var cube = Cube();
